from dotenv import load_dotenv
import os
import json
import inspect

calling_script_dir = os.path.dirname(os.path.abspath(inspect.stack()[1].filename))
env_path = os.path.join(calling_script_dir, '.monit')
load_dotenv(dotenv_path=env_path)

# Code info
project = os.getenv('PROJECT')
company = os.getenv('COMPANY')
dev = os.getenv('DEV')

# Database info
user = os.getenv('DB_USER')
password = os.getenv('DB_PASSWORD')
host = os.getenv('DB_HOST')
database = os.getenv('DB_DATABASE')

db_url = f'mysql+pymysql://{user}:{password}@{host}/{database}'

# Email info
email = os.getenv('EMAIL')
email_password = os.getenv('EMAIL_PASSWORD')

if not project or not company or not dev:
    raise Exception("Por favor, as informações do código não podem ficar em branco.")

if not user or not password or not host or not database:
    raise Exception("Por favor, as informações do banco de dados não podem ficar em branco.")



from datetime import datetime
import sys
import os

from monit import config
from monit import func
from monit.database import Database, Table
from monit.verify_env import verify_env

# Ensure .env is loaded
verify_env()

# Get the initial time
INIT_TIME = datetime.now()

class Monitor:
    TYPE = list()
    ERROR = list()

    def register(self, type=None, error=None):
        table = func.build_table(type, error, Table(), INIT_TIME)
        Database.insert(table)

    def end(self):
        # type = ', '.join(self.TYPE)
        # error = ', '.join(self.ERROR)
        self.register(str(self.TYPE), str(self.ERROR))

    def notify(self, type=None, error=None):
        self.TYPE.append(type)
        self.ERROR.append(error)

    @staticmethod
    def notify_and_exit(type=None, error=None):
        Monitor().register(type, error)
        sys.exit(1)



SetupError="SetupError"
DatabaseError="DatabaseError"
HTTPError="HTTPError"
FileError="FileError"
FolderError="FolderError"
TooManyRequests="TooManyRequests"
DataCreateError="DataCreateError"
DataUpdateError="DataUpdateError"




import logging
import os
from logging.handlers import TimedRotatingFileHandler
import sys
# import io


class LoggingConfigurator:
    # Classe responsável por configurar o logger.

    @staticmethod
    def get_logger(log_level):
        logger = logging.getLogger()
        if not logger.handlers:  # Configure o logger apenas se os manipuladores ainda não estiverem configurados
            logger.setLevel(log_level)

            console_handler = logging.StreamHandler(sys.stdout)
            console_handler.setFormatter(LoggingConfigurator.get_formatter())
            logger.addHandler(console_handler)

            # file_handler = TimedRotatingFileHandler(LoggingConfigurator.get_log_file(), when='midnight', encoding='utf-8')
            # file_handler.setFormatter(LoggingConfigurator.get_formatter())
            # logger.addHandler(file_handler)

            logger.propagate = False
        return logger

    @staticmethod
    def get_formatter():
        # return logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s] %(message)s")
        return logging.Formatter(
            fmt="%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s] %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S"
        )

    @staticmethod
    def get_log_file():
        log_dir = 'logs'
        if not os.path.exists(log_dir):
            os.makedirs(log_dir)
        return os.path.join(os.getcwd(), log_dir, os.path.splitext(os.path.basename(__file__))[0] + '.log')


class Logger:

    @staticmethod
    def debug(message):
        logger = LoggingConfigurator.get_logger(logging.DEBUG)
        logger.debug(message)

    @staticmethod
    def info(message):
        logger = LoggingConfigurator.get_logger(logging.INFO)
        logger.info(message)

    @staticmethod
    def warning(message):
        logger = LoggingConfigurator.get_logger(logging.WARNING)
        logger.warning(message)

    @staticmethod
    def error(message):
        logger = LoggingConfigurator.get_logger(logging.ERROR)
        logger.error(message)

    @staticmethod
    def critical(message):
        logger = LoggingConfigurator.get_logger(logging.CRITICAL)
        logger.critical(message)


# Exemplo de uso:
if __name__ == "__main__":
    log = Logger()

    log.debug('Mensagem de Debug')
    log.info('Mensagem de Informação')
    log.warning('Mensagem de Aviso')
    log.error('Mensagem de Erro')
    log.critical('Mensagem Crítica')

    print("Esta é uma saída padrão.")
    raise ValueError("Esta é uma exceção para testar a saída de erro.")




import socket
import psutil
import time
import datetime
import traceback
from math import floor
import platform
from datetime import datetime
import smtplib
from email.message import EmailMessage

from monit import config


def build_table(type, err, table, init_time):

        table.project = config.project
        table.company = config.company
        table.dev = config.dev
        table.stderr = bool(err)

        fim = datetime.now()

        total_time = fim - init_time
        table.runtime = total_time.total_seconds()
        table.date_init = init_time
        table.date_end = fim

        table.cpu = _get_cpu_usage()
        table.mem = _get_memory_usage()
        table.disk = _get_disk_usage()
        table.ping = _ping_host()
        table.system = platform.system()

        if err:
            error = str(err).replace('\n', '')

            table.type = type
            table.error = error

            _print_error_to_console(type, err)
            _send_email_notification(error)

        return table

def _print_error_to_console(type, err):
    # Imprime o erro no console.
    if err:
        # error_type = type(err).__name__
        tb = traceback.extract_tb(err.__traceback__)
        filename, line, func, text = tb[-1]
        strerror = f"File \"{filename}\", line {line}\n\t{text}\n\n{type}: {err}"
        print(strerror)

def _send_email_notification(err):
    # Envia um e-mail com o erro.
    if config.email and config.email_password:
        message = f"{config.company}\n{config.project}\n\n{err}"

        if err:
            msg = EmailMessage()
            msg['subject'] = f'Error in {config.project} on {config.company}'
            msg['from'] = config.email
            msg['to'] = config.email
            msg.set_content(message)

            with smtplib.SMTP_SSL('smtp.gmail.com', 465) as smtp:
                smtp.login(config.email, config.email_password)
                smtp.send_message(msg)

def _ping_host():
    host = '1.1.1.1'  # Host alvo para ping (por exemplo, google.com)
    try:
        # Cria um socket TCP
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            # Define um tempo limite para a conexão
            s.settimeout(2)
            start_time = time.time()
            # Tenta se conectar ao host na porta 80 (HTTP)
            s.connect((host, 80))
            end_time = time.time()
            # Calcula o tempo de resposta
            rtt = (end_time - start_time) * 1000  # Convertendo para milissegundos
            return f"{round(rtt, 2):.0f}"
    except Exception as e:
        print("Erro ao pingar o host:", e)
        return None

def _get_disk_usage():
    disk = psutil.disk_usage('/')
    total_disk_space = disk.total
    used_disk_space = disk.used
    disk_percent = (used_disk_space / total_disk_space) * 100
    return f"{disk_percent:.0f}%"

def _get_cpu_usage():
    return f"{psutil.cpu_percent(interval=1):.0f}%"

def _get_memory_usage():
    mem = psutil.virtual_memory()
    return f"{mem.percent:.0f}%"

# Função para converter bytes em megabytes
def _bytes_to_mb(bytes):
    return bytes / (1024 * 1024)



import os
import sys
import inspect
from datetime import datetime

class Tee(object):
    def __init__(self, *files):
        self.files = files
    def write(self, obj):
        for f in self.files:
            f.write(obj)
            f.flush()  # Para garantir que a saída seja escrita imediatamente
    def flush(self):
        for f in self.files:
            f.flush()

class Log2File:
    def __init__(self, log_dir="logs"):
        self.log_dir = log_dir
        self.start_logging()

    def start_logging(self):
        if not os.path.exists(self.log_dir):
            os.makedirs(self.log_dir)

        # Obtém o nome do arquivo que importou esta classe
        caller_filename = inspect.stack()[2].filename

        # Define o caminho completo para o arquivo de log
        caller_filename_without_extension = os.path.splitext(os.path.basename(caller_filename))[0]
        log_filename = caller_filename_without_extension + ".log"
        log_path = os.path.join(self.log_dir, log_filename)

        # Abre o arquivo de log em modo de adição para manter o conteúdo existente
        self.log_file = open(log_path, "a")

        # Redireciona a saída padrão (stdout e stderr) para o arquivo de log e para o terminal
        sys.stdout = Tee(sys.stdout, self.log_file)
        sys.stderr = Tee(sys.stderr, self.log_file)

# Inicializa o redirecionamento de log assim que o módulo for importado
# Log2File()

# # Exemplo de uso:
# if __name__ == "__main__":
#     # Seu código aqui...
#     print("Hello, world!")
#     x = 5
#     print("The value of x is:", x)
#     y = "not_an_integer"
#     try:
#         y = int(y)
#     except ValueError as e:
#         print("Erro ao converter para inteiro:", e)

# # O redirecionamento do log será finalizado automaticamente quando o script terminar de ser executado.



from sqlalchemy import create_engine, Column, Integer, String, DateTime, Boolean, Text
# from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from datetime import datetime
from sqlalchemy.orm import declarative_base
from monit import config

Base = declarative_base()

class Table(Base):
    __tablename__ = 'monit'

    id = Column(Integer, primary_key=True)
    project = Column(String(255))
    company = Column(String(255))
    dev = Column(String(255))
    type = Column(String(255))
    stderr = Column(Boolean)
    error = Column(Text)
    runtime = Column(Integer)
    date_init = Column(DateTime)
    date_end = Column(DateTime)
    cpu = Column(String(255))
    mem = Column(String(255))
    disk = Column(String(255))
    system = Column(String(255))
    ping = Column(Integer)

class DatabaseSession:
    def __init__(self, url):
        self.engine = create_engine(url)
        Base.metadata.create_all(self.engine)
        self.Session = sessionmaker(bind=self.engine)

    def get_session(self):
        return self.Session()

class DataManager:
    def __init__(self, db):
        self.db = db
        self.session = self.db.get_session()

    def insert(self, data):
        self.session.add(data)
        self.session.commit()
        return data.id

    def close(self):
            self.session.close()

class Database:
    @staticmethod
    def insert(data):
        db = DatabaseSession(config.db_url)
        data_manager = DataManager(db)
        id = data_manager.insert(data)
        data_manager.close()
        return id



import os

# Função para verificar a existência do arquivo .env
def verify_env():
    if not os.path.exists('.monit'):
        raise FileNotFoundError("MonitError: Arquivo .monit não encontrado. Por favor, crie um arquivo .monit com as configurações necessárias.")
